import { Entity, Column, OneToMany, ManyToOne, PrimaryGeneratedColumn, Generated, Unique, JoinTable, ManyToMany } from "typeorm";
import { IsBoolean, IsEmail, IsInt, IsNotEmpty, IsOptional, IsString, isUUID, } from "class-validator";
import { UserGroup } from './UserGroup';

/**
 * Defines a admin user.
*/
@Entity()
export class User {
  /**
     * autogenerated unique id (primary key).
     */
  @PrimaryGeneratedColumn()
  @IsOptional()
  @IsInt()
  id: number;

  /**
     * autogenerated uuid
     */
  @IsOptional()
  @IsInt()
  @Generated("uuid")
  uuid: string;

  /**
     * user email
     */
  @IsEmail()
  email: string;

  /**
     * username
     */
  @IsString()
  username: string;

  /**
     * firstname
     */
  @IsString()
  @IsNotEmpty()
  firstname: string;

  /**
     * middlename
     */
  @IsString()
  @IsOptional()
  middlename: string;

  /**
     * lastname
     */
  @IsString()
  @IsNotEmpty()
  lastname: string;

  /**
     * password
     */
  @IsString()
  @IsNotEmpty()
  password: string;

  /**
     * userpermissions
     */
  //  TODO: UserPermission implementation
  // @OneToMany(() => UserPermission,userpermission=>)
  // userpermissions: UserPermission[];

  /**
     * groups
     */
  @ManyToMany(() => UserGroup)
  @JoinTable()
  groups: UserGroup[];

  /**
     * is user enabled?
     */
  @IsBoolean()
  enabled: boolean;

  /**
     * jwt refresh count
     */
  @IsInt()
  @Column({ default: 1 })
  refreshTokenCount: number;

  /**
     * profilepic
     */
  @IsString()
  profilepic: string;
}
